package mcintyr.edu.montana.cs.esus;

import java.util.ArrayList;

/**
 * An associative array implementation.
 *
 * @Author  - Kyle McIntyre
 * @Version - 5.26.05
 */

public class AssociativeArray
{
    private ArrayList<Pair> associative; // Stores the array elements

    /**
     * Default constructor. Initializes an array list for storing
     * the Pair objects.
     */
    
    public AssociativeArray()
    {
        associative = new ArrayList<Pair>(); 
    }

    /**
     * Returns the key at the specified index of the array.
     *
     * @return - The key at the specified index of the array.
     */

    public String getKey(int i)
    {
        return associative.get(i).getKey(); 
    }

    /**
     * Returns the value at the specified index of the array.
     *
     * @param i - The index in the array. 
     * @return  - The value at the specified index of the array.
     */

    public String getValue(int i)
    {
        return associative.get(i).getValue(); 
    }

    /**
     * Returns the value corresponding to the specified key.
     * If no such value exists, returns the empty string rather
     * than null.
     *
     * @param key - The key for the key/value pair.
     * @return    - The associated value.
     */

    public String getValue(String key)
    {
        for (int i = 0; i < associative.size(); i++)
        {
            if (associative.get(i).getKey().equals(key))
                return associative.get(i).getValue();
        }
        return ""; 
    }

    /**
     * Sets a new key/value pair in the associative array.
     * If the key already exists, its value is overwritten, otherwise
     * a new Pair object is created and stored in the array.
     *
     * @param key   - The key for key/value pair.
     * @param value - The value for key/value pair.
     */

    public void setValue(String key, String value)
    {
        boolean done = false; 
        for (int i = 0; i < associative.size() && !done; i++)
        {
            if (associative.get(i).getKey().equals(key))
            {
                associative.get(i).setValue(value);
                done = true;
            }
        }
        if (!done)
            associative.add(new Pair(key, value)); 
    }

    /**
     * @return - The number of elements currently in the array.
     */

    public int size()
    {
        return associative.size(); 
    }

    /**
     * Empties the current associative array.
     */

    public void empty()
    {
        associative = new ArrayList(); 
    }
}