package mcintyr.edu.montana.cs.esus;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData; 

/**
 * A class that contains utility functions, such as 
 * form validators and synchronization functions. 
 *
 * @Author  - Kyle McIntyre
 * @Version - 5.25.05
 */

public class Utilities
{
    private DatabaseConnection db; // An active database connection

    public Utilities()
    {
        db = new DatabaseConnection();  
    }

    
    /**
     * This function calls every known validation function to ensure
     * that a form's post data is of the correct nature to be entered
     * into the database. The function ignores forms that blank so that 
     * the function can be called from any context (user submission, product
     * modification, etc). 
     *
     * @param forms - The associative array containing the form data.
     * @return      - Whether or not all of the tests were passed. 
     */
     
    
    public boolean checkForms(AssociativeArray forms)
    {
	    boolean valid = true;
	    if (!forms.getValue("date").equals(""))
	        valid &= checkDateValidity(forms.getValue("date"));
            if (!forms.getValue("due_date").equals(""))
	        valid &= checkDateValidity(forms.getValue("due_date"));
	    if (!forms.getValue("phone").equals(""))
	        valid &= checkPhoneValidity(forms.getValue("phone"));
	    if (!forms.getValue("email").equals(""))
	        valid &= checkEmailValidity(forms.getValue("email"));
            if (!forms.getValue("product_id").equals(""))
	        valid &= checkIntValidity(forms.getValue("product_id"));
            if (!forms.getValue("user_id").equals(""))
	        valid &= checkIntValidity(forms.getValue("user_id"));
            if (!forms.getValue("quantity").equals(""))
	        valid &= checkIntValidity(forms.getValue("quantity"));
            if (!forms.getValue("in_stock").equals(""))
	        valid &= checkIntValidity(forms.getValue("in_stock"));
            if (!forms.getValue("rentals").equals(""))
	        valid &= checkIntValidity(forms.getValue("rentals"));
            if (!forms.getValue("fee_value").equals(""))
	        valid &= checkFloatValidity(forms.getValue("fee_value"));
            if (!forms.getValue("fee").equals(""))
	        valid &= checkFloatValidity(forms.getValue("fee"));

	    return valid; 
    }
    
    public boolean checkIntValidity(String str)
    {
        return str.matches("\\d*");
    }

    public boolean checkFloatValidity(String str)
    {
        return str.matches("\\d*\\.\\d*");
    }

    public boolean checkDateValidity(String date)
    {
	return date.matches("[\\d]{4}-[\\d]{2}-[\\d]{2}");
    }
    
    public boolean checkPhoneValidity(String phone_number)
    {
	return phone_number.matches("[\\d]{3}-[\\d]{3}-[\\d]{4}");
    }
    
    public boolean checkEmailValidity(String email)
    {
        return email.matches(".*@.*\\..+");
    }

    /**
     * Checks to see if a string is null or empty, and if so 
     * returns true.
     *
     * @param str - The string to be checked.
     * @return    - Whether or not the string null/empty. 
     */

    public boolean isNull(String str)
    {
        if (str == null || str.equals(""))
            return true;
        else
            return false;
    }

    /**
     * Checks to see if the specified login already exists in
     * in the database.
     *
     * @param login - The login value to be compared against the database.
     * @return      - Whether or not the login has been used previously.
     */
    
    public boolean isLoginUnique(String login)
    {
        String query = "SELECT login FROM users WHERE login = '" + login + "';";
        ResultSet result = db.query(query);
        try 
        {
            if (result.next())
                return false;
            else
                return true;
        }
        catch (Exception e) {
            return false;
        }
    }

    /**
     * A function that can be called whenever a User object needs
     * to be resynced with values in the database.
     *
     * @param user - The user object to be updated. 
     */

    public void updateUser(User user)
    {
        try 
        {
            // Query database for user information
            String query = "SELECT * FROM users WHERE login = '" + user.getValue("login") + "'";
            ResultSet result = db.query(query);
            result.next();
 
            ResultSetMetaData rsmd = result.getMetaData();
            for (int i = 1; i <= rsmd.getColumnCount(); i++)
                user.setValue(rsmd.getColumnName(i), result.getString(rsmd.getColumnName(i)));
        }
        catch (Exception e) 
        {
            System.out.println(e.getMessage());
        }
    }
}