<%
/**
 * A file for listing the available products. Generates dynamic SQL
 * queries based on the user's requests. 
 *
 * @Author  - Kyle McIntyre
 * @Version - 5.26.05
 */
%>

<%@ page import="java.sql.ResultSet"         %>
<%@ page import="java.sql.ResultSetMetaData" %>
<%@ page import="java.util.StringTokenizer"  %>
<%@ page import="java.text.NumberFormat"     %>

<jsp:useBean id="util"   class="mcintyr.edu.montana.cs.esus.Utilities"          scope="session" />
<jsp:useBean id="db"     class="mcintyr.edu.montana.cs.esus.DatabaseConnection" scope="session" />
<jsp:useBean id="user"   class="mcintyr.edu.montana.cs.esus.User"               scope="session" />
<jsp:useBean id="form"   class="mcintyr.edu.montana.cs.esus.AssociativeArray"   scope="page"    />

<%!
    // Page variables
    private int start_index  = 1;
    private int num_results  = 5;
    private int result_size  = 0;
    private String cart = "";

    /**
     * A simple function for returning GET/POST variables that
     * ensures the result of a request is never null.
     * 
     * @param request - The request object.
     * @param name    - The GET/POST variable name.
     * @return - The resulting value, or the empty string if no value was found. 
     */ 
    
    private String getParameter(HttpServletRequest request, String name)
    {
        String value = request.getParameter(name);
        if (value == null)
            return "";
        else
            return value;     
    }

    /**
     * A function that generates the necessary HTML markup for the 
     * shopping cart icon/links on the listing page. The markup varies
     * when the user already has the item in his or her cart, or the item
     * is out of stock.
     *
     * @param product_id - The product to generate cart markup for.
     * @param in_stock   - Number of copies in stock.
     * @return - Corresponding HTML markup. 
     */

    private String getCartMarkup(String product_id, int in_stock)
    {
       
        if (in_stock <= 0)
        {
            // Product out of stock, prevent addition to shopping cart.
            return "<img src=\"/movies/images/no_cart.gif\" alt=\"Item out of stock\" />";
        }
        else
        {
            StringTokenizer toke = new StringTokenizer(cart, ",");
            boolean full = false;
            while (toke.hasMoreTokens())
            {
                if (product_id.equals(toke.nextToken()))
                    full = true;
            }
         
            if (full)
                return "<img src=\"/movies/images/cart_full.gif\" alt=\"Already in cart\" />";
            else
                return "<a href=\"/movies/enduser/add.jsp?product_id=" + product_id + "\"><img src=\"/movies/images/cart.gif\" border=\"0\" alt=\"Add to Cart\" /></a>"; 
        }   
    }
%>

<%@ include file="/movies/include/header.tpl"     %>
<%@ include file="/movies/include/navigation.jsp" %>

<%
    String query = "";
    ResultSet result = null;

    // Create a number formatter for the price listings. 
    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumFractionDigits(2);
    nf.setMaximumFractionDigits(2);

    // Extract the user's current shopping cart. 
    cart = user.getValue("cart");

    if (request.getParameter("latest") != null)
        out.println("<b>Latest Releases</b><br />");
%>

<form action="/movies/common/list.jsp" method="post">
Media Type:
<select name="type">
<%
    // Extract all media types from the media types table
    query  = "SELECT * FROM media_types";
    result = db.query(query);

    while (result.next())
    {
        String type_name = result.getString("type_name");
%>
  <option value="<%= type_name %>" <% if (getParameter(request, "type").equals(type_name)) { out.println("selected=\"true\"");} %>><%= type_name %></option>
<%
    }
%>
  <option value=""          <% if (getParameter(request, "type").equals("")) { out.println("selected=\"true\"");} %>>Any</option>
</select>
Genre:
<select name="genre">
<%
       // Extract all genres from the genre table
       query  = "SELECT * FROM genres ORDER BY genre_name";
       result = db.query(query);

       while (result.next())
       {
           String genre_name = result.getString("genre_name");
%>
  <option value="<%= genre_name %>" <% if (getParameter(request, "genre").equals(genre_name)) { out.println("selected=\"true\"");} %>><%= genre_name %></option>
<%
       }
%>
  <option value=""          <% if (getParameter(request, "genre").equals("")) { out.println("selected=\"true\"");} %>>Any</option>
</select>
Rated: 
<select name="rated">
<%
       // Extract all rating types from the ratings table
       query  = "SELECT * FROM ratings";
       result = db.query(query);

       while (result.next())
       {
           String rating_name = result.getString("rating_name");
%>
  <option value="<%= rating_name %>" <% if (getParameter(request, "rated").equals(rating_name)) { out.println("selected=\"true\"");} %>><%= rating_name %></option>
<%
       }
%>
  <option value="" <% if (getParameter(request, "rated").equals("")) { out.println("selected=\"true\"");} %>>Any</option>
</select>
<input name="submit" type="submit" value="List" />
</form>

<div class="page">

<%
    // Generate the product listing

    String type      = request.getParameter("type");
    String genre     = request.getParameter("genre");
    String rated     = request.getParameter("rated");
    
    query = "SELECT product_id, title, type, genre, rated, in_stock, f.fee_value FROM products p, fees f ";
    query += "WHERE f.fee_name = p.fee_type AND"; 
    
    if (!util.isNull(type))
        query += " type = '" + type + "' AND";

    if (!util.isNull(genre))
        query += " genre LIKE('%" + genre + "%') AND";

    if (!util.isNull(rated))
        query += " rated = '" + rated + "' AND";

    // Clear out erroneous AND
    query = query.substring(0, query.length() - 4);

    if (request.getParameter("latest") != null)
        query += " ORDER BY date DESC;";
    else
        query += " ORDER BY title;";

    result = db.query(query);
    result_size = 0;
    while (result.next())
        result_size++;
   
    String nav = request.getParameter("nav");
    if (nav == null)
        nav = "new";
    if (nav.equals("prev"))
        start_index = (int)Math.max(start_index - num_results, 1);
    else if (nav.equals("next"))
        start_index = (int)Math.min(start_index + num_results, result_size);
    else
        start_index = 1;
    
    // Output the listing
    ResultSetMetaData rsmd = result.getMetaData();
    int count = start_index; 
 
    out.println("<table width=\"100%\">");
    while (result.absolute(count) && count < start_index + num_results)
    {
        count++; 
        %>
              <tr>
                <td colspan="4" align="left">
                  <%= getCartMarkup(result.getString("product_id"), result.getInt("in_stock")) %> 
                  <b>
                    <a href="/movies/common/listing.jsp?product_id=<%= result.getInt("product_id") %>">
                      <%= result.getString("title") %> [<%= result.getString("type").toUpperCase() %>]
                    </a>
                  </b> - <i>(<%= result.getString("genre") %>)</i></td>
              </tr>
              <tr>
                <td width="5%"></td>
                <td>Rated <%= result.getString("rated") %></td>
                <td>In Stock: <%= result.getInt("in_stock") %></td>
                <td>Rental Cost: <%= '$' + nf.format(result.getFloat("f.fee_value")) %></td>
              </tr>
              <tr>
                <td colspan="4"><br /></td>
              </tr>
        <%
    }
    out.println("</table>");
    count--;
  
%>
Total Results: <%= result_size %><br />
Currently viewing results <%= start_index %> through <%= count %>: 
<% 
    // Navigation links
    if (start_index > 1) 
    {
        %> <a href="/movies/common/list.jsp?nav=prev">Previous</a> &nbsp; <% 
    }
    if (count < result_size) 
    {
        %> <a href="/movies/common/list.jsp?nav=next">Next</a> <%
    }
%>
  
</div>

<%@ include file="/movies/include/footer.tpl"    %>
